@@ -2,14 +2,14 @@
 from dataclasses import dataclass, field
 from typing import Optional
 
-from hundredandten.actions import (Action, Bid, Discard, Play, SelectTrump,
-                                   Unpass)
+from hundredandten.actions import (Action, Bid, DetailedDiscard, Discard, Play,
+                                   SelectTrump, Unpass)
 from hundredandten.constants import (TRICK_VALUE, BidAmount, RoundRole,
                                      RoundStatus, SelectableSuit)
 from hundredandten.decisions import (best_card, desired_trump, max_bid,
                                      non_trumps, trumps, worst_card,
                                      worst_card_beating)
-from hundredandten.deck import Deck
+from hundredandten.deck import Card, Deck
 from hundredandten.events import Event, TrickEnd, TrickStart
 from hundredandten.group import Group, Player
 from hundredandten.hundred_and_ten_error import HundredAndTenError
@@ -22,7 +22,7 @@ class Round:
     players: Group[Player] = field(default_factory=Group)
     bids: list[Bid] = field(default_factory=list)
     selection: Optional[SelectTrump] = None
-    discards: list[Discard] = field(default_factory=list)
+    discards: list[DetailedDiscard] = field(default_factory=list)
     tricks: list[Trick] = field(default_factory=list)
     deck: Deck = field(default_factory=Deck)
 
@@ -141,7 +141,9 @@ class Round:
                 TrickStart(),
                 *trick.plays,
                 # don't include the trick end event if it hasn't ended
-                *([TrickEnd(trick.winning_play.identifier)] if trick.winning_play else [])
+                *([TrickEnd(trick.winning_play.identifier)]
+                  if (trick.winning_play and len(trick.plays) == len(self.players))
+                  else [])
             ]
             for trick in self.tricks]
 
@@ -152,7 +154,7 @@ class Round:
             *[trick_event for event_list in trick_events for trick_event in event_list]
         ]
 
-    @ property
+    @property
     def scores(self) -> list[Score]:
         '''
         The scores each player earned for this round
@@ -202,23 +204,36 @@ class Round:
 
         return base_scores
 
-    def bid(self, bid: Bid) -> None:
+    def act(self, action: Action) -> None:
+        '''Perform an action as a player of the game'''
+        if isinstance(action, Bid):
+            self.__bid(action)
+        if isinstance(action, Unpass):
+            self.__unpass(action)
+        if isinstance(action, SelectTrump):
+            self.__select_trump(action)
+        if isinstance(action, Discard):
+            self.__discard(action)
+        if isinstance(action, Play):
+            self.__play(action)
+
+    def __bid(self, bid: Bid) -> None:
         '''Record a bid from a player'''
         identifier = bid.identifier
         amount = bid.amount
         if self.status == RoundStatus.BIDDING and self.active_player == self.players.by_identifier(
                 identifier):
-            self.__bid(identifier, amount)
+            self.__handle_bid(identifier, amount)
         elif amount == BidAmount.PASS:
             self.players.add_role(identifier, RoundRole.PRE_PASSED)
         else:
             raise HundredAndTenError("Cannot bid out of order")
 
-    def unpass(self, unpass: Unpass) -> None:
+    def __unpass(self, unpass: Unpass) -> None:
         '''Discount a prepass bid from the identified player'''
         self.players.remove_role(unpass.identifier, RoundRole.PRE_PASSED)
 
-    def select_trump(self, select_trump: SelectTrump) -> None:
+    def __select_trump(self, select_trump: SelectTrump) -> None:
         '''Select the passed suit as trump'''
         if self.status != RoundStatus.TRUMP_SELECTION:
             raise HundredAndTenError("Cannot select trump outside of the trump selection phase.")
@@ -227,7 +242,7 @@ class Round:
 
         self.selection = select_trump
 
-    def discard(self, discard: Discard) -> None:
+    def __discard(self, discard: Discard) -> None:
         '''
         Discard the selected cards from the identified player's hand and replace them
         '''
@@ -238,13 +253,15 @@ class Round:
         if any(card not in self.active_player.hand for card in discard.cards):
             raise HundredAndTenError("You may only discard cards that are in your hand.")
 
-        self.active_player.hand = list(
+        remaining = list(
             filter(lambda c: c not in discard.cards, self.active_player.hand))
+
+        self.active_player.hand = [*remaining]
         self.active_player.hand.extend(self.deck.draw(len(discard.cards)))
-        self.discards.append(discard)
+        self.discards.append(DetailedDiscard(discard.identifier, discard.cards, remaining))
         self.__end_discard()
 
-    def play(self, play: Play) -> None:
+    def __play(self, play: Play) -> None:
         '''Play the specified card from the identified player's hand'''
 
         active_player_trump_cards = trumps(self.active_player.hand, self.trump)
@@ -278,6 +295,13 @@ class Round:
             if self.__is_available_bid(identifier, bid_amount)
         ]
 
+    def original_hand(self, identifier: str) -> list[Card]:
+        '''Return the identified player's original hand'''
+        player = self.players.find_or_use(Player(identifier))
+        discard = next((d for d in self.discards if d.identifier == identifier), None)
+
+        return discard.cards + discard.kept if discard else player.hand
+
     def suggestion(self) -> Action:
         '''Return the suggested action given the game state'''
         if self.status == RoundStatus.BIDDING:
@@ -335,7 +359,7 @@ class Round:
 
         return Play(self.active_player.identifier, card)
 
-    def __bid(self, identifier: str, amount: BidAmount) -> None:
+    def __handle_bid(self, identifier: str, amount: BidAmount) -> None:
         if amount in self.available_bids(identifier):
             self.bids.append(Bid(identifier, amount))
             self.__handle_prepass()
@@ -345,7 +369,7 @@ class Round:
     def __handle_prepass(self) -> None:
         if self.status == RoundStatus.BIDDING and RoundRole.PRE_PASSED in self.active_player.roles:
             self.players.remove_role(self.active_player.identifier, RoundRole.PRE_PASSED)
-            self.__bid(self.active_player.identifier, BidAmount.PASS)
+            self.__handle_bid(self.active_player.identifier, BidAmount.PASS)
 
     def __is_available_bid(self, identifier: str, amount: BidAmount) -> bool:
         '''Determine if the listed bid amount is available to the listed player'''
